module.exports = {
  description: 'returns the same module instance if requiring with and without extension',
  pluginOptions: {
    dynamicRequireTargets: ['fixtures/function/dynamic-require-extensions/submodule.js']
  }
};
