module.exports = {
  description: 'resolves both windows and posix absolute paths',
  pluginOptions: {
    dynamicRequireTargets: ['fixtures/function/dynamic-require-absolute-paths/submodule.js']
  }
};
