global.sideEffects = global.sideEffects || [];
global.sideEffects.push('array-index');

export {value as value1} from './dep1.js';
export {value as value2} from './dep2.js';
export {value as value3} from './dep3-effect.js';
export {value as value4} from './nested/dep4';
export {value as value5} from './nested/dep5-effect';
